#!/usr/bin/perl
#############################################################################
## Fullizer - the full-report program for Webalizer by Bradford L. Barrett ##
## Version: 0.1 1999-07-20                                                 ##
## This programm is written by Stanislav Malyshev <frodo@sharat.co.il>     ##
## (C) 1999                                                                ##
## You can find Webalizer at http://www.mrunix.net/webalizer/              ##
#############################################################################
##    This program is free software; you can redistribute it and/or modify ##
##    it under the terms of the GNU General Public License as published by ##
##    the Free Software Foundation; either version 2 of the License, or    ##
##    (at your option) any later version, and provided that the above      ##
##    copyright and permission notice is included with all distributed     ##
##    copies of this or derived software.                                  ##
##                                                                         ##
##    This program is distributed in the hope that it will be useful,      ##
##    but WITHOUT ANY WARRANTY; without even the implied warranty of       ##
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        ##
##    GNU General Public License for more details.                         ##
##                                                                         ##
##    You should have received a copy of the GNU General Public License    ##
##    along with this program; if not, write to the Free Software          ##
##    Foundation, Inc., 59 Temple Place -                                  ##
##    Suite 330, Boston, MA 02111-1307, USA                                ##
#############################################################################

use FileHandle;
use Getopt::Std;

sub usage {
    print <<"ENDU";
Usage: $0 -d dir [-o dir] [-c file] [-i dir]
\t-d input directory (with webalyzer.current and usage_)
\t-o output directory defaults to input/lists 
\t-c another name for webalizer.current
\t-i insert links to the files into usage stats with directory prefix 
ENDU
    ;
    exit;
}

getopt("odci",\%opts);

&usage() unless defined $opts{'d'};

$in_dir = $opts{'d'};
$hist_file = $opts{'c'} || "webalizer.current";
$out_dir = $opts{'o'} || "$in_dir/lists";

open(HIST,"$in_dir/$hist_file") || die "Cannot open current file $in_dir/$hist_file: $!";
$head = <HIST>;

# Webalizer V1.30-04 Incremental Data - 07/19/1999 22:31:24

($mon,undef,$year) = ($head =~ m{(\d+)/(\d+)/(\d+)});

## try to find webalizer files and fetch headers
$header="";
if(-f "$in_dir/usage_$year$mon.html") {
    open(USAGE,"<$in_dir/usage_$year$mon.html");
    while(<USAGE>) {
	$header .= $_;
	last if /^<hr>/i;
    }
}

if($opts{'i'}) {
    ## find where to insert link
    while(<USAGE>) {
	last if /^<hr>/i;
    }
    $place = USAGE->getpos;
    $usg_tail="";
    $client_mark = "<! -- client -->";
    $files_mark = "<! -- files -->";
    while(<USAGE>) {
	$skip_client = 1 if(/$client_mark/o);
	$skip_files = 1 if(/$files_mark/o);
	$usg_tail .= $_;
    }
    if(!$skip_client || !$skip_files) {
	close(USAGE);
	open(USAGE,"+<$in_dir/usage_$year$mon.html");
	USAGE->setpos($place);
	print USAGE "$client_mark\n<a href=\"$opts{'i'}/sites_$year$mon.html\">Full statistics by clients</a><br>\n" unless $skip_client;
	print USAGE "$files_mark\n<a href=\"$opts{'i'}/files_$year$mon.html\">Full statistics by files</a><br>\n" unless $skip_files;
	print USAGE $usg_tail;
    }
}

close(USAGE);

## Sites Listing

$out_sites = $out_dir . "/sites_" . $year . $mon . ".html";
$out_files = $out_dir . "/files_" . $year . $mon . ".html";
open(OUTS,">$out_sites") || die "Cannot open sites file $out_sites: $!";
open(OUTF,">$out_files") || die "Cannot open files file $out_files: $!";

## find montly sites stats
while(<HIST>) {
    last if /# -sites- \(monthly\)/;
}

while(<HIST>) {
    last if /# End/;
    chomp;
    $site = $_;
    $_ = <HIST>;
    ($flag,$count,$files,$xfer,$visit) = split(/\s+/);
    $counter{$site}=$count;
    $counter_sum += $count;
    $files_counter{$site}=$files;
    $files_sum += $files;
    $xfer_counter{$site}=$xfer;
    $xfer_sum += $xfer;
    $visit_counter{$site}=$visit;
    $visit_sum += $visit;
    $_ = <HIST>;
}

print OUTS <<"ENDH";
$header
<br clear=all></center>
<h3>Total Transfers By Sites</h3>
<pre>
     Requests       Files      Bytes Transferred     Visits    | URL   
----------------------------------------------------------------------------------------------
ENDH
    ;

format File_List =
 @>>>>> @##.#% @>>>>> @##.#% @>>>>>>>>>> @##.#% @>>>>> @##.#%  | @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 $counter{$site}, $counter{$site}/$counter_sum*100, $files_counter{$site}, $files_counter{$site}/$files_sum*100, $xfer_counter{$site}, $xfer_counter{$site}/$xfer_sum*100, $visit_counter{$site},$visit_counter{$site}/$visit_sum*100, $site
.

OUTS->format_name("File_List");
foreach $site (sort { $counter{$b} <=> $counter{$a} } keys %counter) {
    write OUTS;
}

print OUTS "----------------------------------------------------------------------------------------------\n";

format Sum_List = 
 @>>>>>        @>>>>>        @>>>>>>>>>>        @>>>>>       
 $counter_sum, $files_sum,   $xfer_sum,         $visit_sum
==============================================================================================
.

OUTS->format_name("Sum_List");
write OUTS;

print OUTS "</pre>\n</body></html>";

## Sites Listing done

## Start Files Listing

%counter=();
%files_counter=();
%xfer_counter=();
%visit_counter=();
$counter_sum=0;
$files_sum=0;
$xfer_sum=0;
$visit_sum=0;
## find montly sites stats
while(<HIST>) {
    last if /# -urls-/;
}

while(<HIST>) {
    last if /# End/;
    chomp;
    $url = $_;
    $_ = <HIST>;
    ($flag,$count,$files,$xfer,$visit) = split(/\s+/);
    $counter{$url}=$count;
    $counter_sum += $count;
    $files_counter{$url}=$files;
    $files_sum += $files;
    $xfer_counter{$url}=$xfer;
    $xfer_sum += $xfer;
    $visit_counter{$url}=$visit;
    $visit_sum += $visit;
}

print OUTF <<"ENDH";
$header
<br clear=all></center>
<h3>Total Transfers By Files</h3>
<pre>
     Requests    Bytes Transferred     Visits    | URL   
----------------------------------------------------------------------------------------------
ENDH
    ;

format File_List2 =
 @>>>>> @##.#% @>>>>>>>>>> @##.#% @>>>>> @##.#%  | @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 $counter{$url}, $counter{$url}/$counter_sum*100, $xfer_counter{$url}, $xfer_counter{$url}/$xfer_sum*100, $visit_counter{$url},$visit_counter{$url}/$visit_sum*100, $url
.

OUTF->format_name("File_List2");
foreach $url (sort { $counter{$b} <=> $counter{$a} } keys %counter) {
    write OUTF;
}

print OUTF "----------------------------------------------------------------------------------------------\n";

format Sum_List2 = 
 @>>>>>        @>>>>>>>>>>        @>>>>>       
 $counter_sum, $xfer_sum,         $visit_sum
==============================================================================================
.

OUTF->format_name("Sum_List2");
write OUTF;

print OUTF "</pre>\n</body></html>";
